# Badge Reader binding usage

## I - Configure Network

Be care to be able to ping your device. Indeed, We put reader with a row IP like : 172.25.50.xx. So, you will need to add the network interface:

```bash
sudo ip address add 172.25.50.124/24 dev enp0s31f6 
```

## II - Binding Configuration

This project is actually coded in V3 using the controller. Indeed, it will be mandatory to provides a config file in JSON to configure the binding and this badge readers plugins ! This config need to have the **metadata** information:

```json
{
"id": "http://iot.bzh/download/public/schema/json/ctl-schema.json#",
"$schema": "http://json-schema.org/draft-04/schema#",
"description": "Controller configuration schema",

"metadata": {
    "uid": "badge-reader-binding",
    "version": "1.0",
    "api": "badge-reader",
    "info": "Default config to use badge-reader binding"
},
"plugins": [],
"family-reader": [],
}
```

### a. plugins configuration

The plugin configuration has to provides a list of all necessary information to load the correct plugin. Here an example which load the cn56 plugin:

```json
{
  "uid": "cn56",
  "info": "Plugin to handle cn56 reader",
  "spath": "package/lib/:./lib/plugins:./var",
  "libs": [
  "cn56.ctlso"
  ]
}
```

* **uid**: The Unique Identification of the plugin
* **info**: Summary of the plugin (optional)
* **spath**: relative path where to search the plugins
* **libs**: name of the shared library

### b. Family reader configuration

The family-reader configuration is here to configure all kind of plugins. For example, if we put the cn56 plugin we have to configure all reader which will be present !
Here is the example if we have 2 readers that we will regroup into one family:

```json
{
  "uid": "cn56",
  "info": "CN56 Badge Reader TCP",
  "mode": "manual",
  "format": "plugin://cn56#doors",
  "readers": [
    {
      "uid": "cn56-ex-sas",
      "info": "Old badge reader principal entry",
      "reader": "cn56-ex-sas",
      "tcp_address": "172.25.50.39",
      "tcp_port": 2000,
      "mode": "auto",
      "verbose": 0,
      "config": {
        "blockToRead": 0,
        "csnRead": 0,
        "beepLedEnable": 0,
        "ledBuzzerControle": 0,
        "readerEnable": 1,
        "requestIdle": 1,
        "uartOutEnable": 1,
        "openWiegand": 0,
        "setWiegand": 1
      },
      "keys": [
        {"uid":"default-A", "idx": 0, "value":["0xFF","0xFF","0xFF","0xFF","0xFF","0xFF"]},
        {"uid":"key-a", "idx": 0, "value":["0x01","0x02","0x03","0x04","0x05","0x06"]},
        {"uid":"key-b", "idx": 1, "value":["0x0A","0x0B","0x0C","0x0D","0x0E","0x0F"]}
      ],
      "cmds": [
        {"uid":"read-magic-key-1" , "group": 0, "action":"read", "sec": 11, "key": "key-a", "blk":1, "len":5},
        {"uid":"read-magic-key-2" , "group": 0, "action":"read", "sec": 11, "key": "key-a", "blk": 2, "len":9},
        {"uid":"verify-data-control" , "group": 0, "action":"read", "sec": 11, "key": "key-a", "blk": 0, "len":8}
      ]
    },{
      "uid": "cn56-revision",
      "info": "Badge reader going to revision",
      "reader": "cn56-revision",
      "tcp_address": "172.25.50.24",
      "tcp_port": 2000,
      "mode": "auto",
      "rank": "admin",
      "verbose": 0,
      "config": {
        "blockToRead": 0,
        "csnRead": 0,
        "beepLedEnable": 0,
        "ledBuzzerControle": 0,
        "readerEnable": 1,
        "requestIdle": 1,
        "uartOutEnable": 1,
        "openWiegand": 0,
        "setWiegand": 1
      },
      "keys": [
        {"uid":"default-A", "idx": 0, "value":["0xFF","0xFF","0xFF","0xFF","0xFF","0xFF"]},
        {"uid":"key-a", "idx": 0, "value":["0x01","0x02","0x03","0x04","0x05","0x06"]},
        {"uid":"key-b", "idx": 1, "value":["0x0A","0x0B","0x0C","0x0D","0x0E","0x0F"]}
      ],
      "cmds": [
        {"uid":"trailer" , "group": 0, "action":"trailer", "sec": 11, "key": "default-A", "trailer": {"keyA": "key-a", "keyB": "key-b", "acls": ["0x79","0x67","0x88","0x00"]}},
        {"uid":"write-magic-key-1" , "group": 0, "action":"write", "sec": 11, "key": "key-b", "blk": 1, "len":5, "data": "magic"},
        {"uid":"write-magic-key-2" , "group": 0, "action":"write", "sec": 11, "key": "key-b", "blk": 2, "len":9, "data": "encrypted"},
        {"uid":"verify-data-control" , "group": 0, "action":"read", "sec": 11, "key": "key-a", "blk": 0, "len":8}
      ]
    }
  ] 
}
```

#### 1. family reader

* **mode**: can be **manual**/**auto**. In **auto** mode all readers of this family will execute their command if they catch a good badge following all [security](./4-security.md)
* **format**: Inform about which plugins readers comes and with which behavior readers has to be ! Write like this: **plugin://NAME_PLUGIN#BEHAVIOR_PLUGIN** The behavior will be defined into the plugins code.
* **readers**: A JSON array which list all reader of this family

#### 2. Reader config (example of CN56 plugin)

* **uid**: UID of the reader
* **info**: Summary
* **reader**: Name of the reader
* **tcp_address**
* **tcp_port**
* **mode**: Same explanation from family-reader config
* **rank**: if Admin is specified, it will have the administrator behavior
* **verbose**: to print verbose in debug
* **config**: Special config for CN56 reader
* **keys**: List of the keys tha the reader will uses for this command
* **cmds**: The list of command that the reader will automatically do

## III - Use the binding

Go to the build directory and run:

```bash
afb-binder --name=afb-badge-reader --port=1234 --workdir=./package/lib --binding=afb-badge-reader.so -vvv
```

Output will be:

```bash
------BEGIN OF CONFIG-----
-- {
--    "name": "afb-badge-reader",
--    "port": 1234,
--    "workdir": "./package/lib",
--    "binding": [
--      {
--        "path": "afb-badge-reader.so"
--      }
--    ],
--    "apitimeout": 20,
--    "cache-eol": 100000,
--    "cntxtimeout": 32000000,
--    "session-max": 200,
--    "jobs-max": 200,
--    "threads-max": 5,
--    "uploaddir": ".",
--    "rootbase": "/opa",
--    "rootapi": "/api",
--    "rootdir": ".",
--    "log": "error+warning+notice+info+debug"
--  }
------END OF CONFIG-----
------BEGIN OF CONFIG-----
-- {
--    "name": "afb-badge-reader",
--    "port": 1234,
--    "workdir": "./package/lib",
--    "binding": [
--      {
--        "path": "afb-badge-reader.so"
--      }
--    ],
--    "apitimeout": 20,
--    "cache-eol": 100000,
--    "cntxtimeout": 32000000,
--    "session-max": 200,
--    "jobs-max": 200,
--    "threads-max": 5,
--    "uploaddir": ".",
--    "rootbase": "/opa",
--    "rootapi": "/api",
--    "rootdir": ".",
--    "log": "error+warning+notice+info+debug"
--  }
------END OF CONFIG-----
INFO: running with pid 52713
INFO: API monitor added
INFO: Can't connect supervision socket to @urn:AGL:afs:supervision:socket: Connection refused
NOTICE: Browser URL= http://localhost:1234
NOTICE: Serving rootdir=. uploaddir=.
INFO: binding [afb-badge-reader.so] looks like an AFB binding V3
NOTICE: [API (null)] Controller in afbBindingEntry
NOTICE: [API (null)] Using default environnement config path : /home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/conf.d/project/etc:/home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/build/package/etc:/usr/local/badge-reader-binding/etc
NOTICE: [API (null)] Json config directory : /home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/conf.d/project/etc:/home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/build/package/etc:/usr/local/badge-reader-binding/etc:/home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/build/package/lib
DEBUG: [API (null)] CONFIG-SCANNING dir=/usr/local/badge-reader-binding/etc not readable
WARNING: [API (null)] CTL-INIT JSON file found but not used : /home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/build/package/etc/control-badge-reader.json [/usr/src/packages/BUILD/ctl-lib/ctl-config.c:93,ConfigSearch]
NOTICE: [API (null)] api will be using config : /home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/conf.d/project/etc/control-badge-reader.json
INFO: [API (null)] CTL-LOAD-CONFIG: loading config filepath=/home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/conf.d/project/etc/control-badge-reader.json
NOTICE: [API (null)] Controller API='badge-reader' info='Default config to use badge-reader binding'
INFO: API badge-reader added
DEBUG: [API badge-reader] Plugin search path : '/home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/build/package/lib/package/lib/:/home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/build/package/lib/../package/lib/:/home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/build/package/lib/./lib/plugins:/home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/build/package/lib/.././lib/plugins:/home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/build/package/lib/./var:/home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/build/package/lib/.././var'
DEBUG: [API (null)] CONFIG-SCANNING dir=/home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/build/package/lib/package/lib/ not readable
DEBUG: [API (null)] CONFIG-SCANNING dir=/home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/build/package/lib/../package/lib/ not readable
DEBUG: [API (null)] CONFIG-SCANNING dir=/home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/build/package/lib/./lib/plugins not readable
DEBUG: [API (null)] CONFIG-SCANNING dir=/home/val/workspace/src/redpesk_OS_Common_Components/badge-reader-binding/build/package/lib/./var not readable
NOTICE: [API badge-reader] CTL-PLUGIN-LOADONE cn56-reader successfully registered
DEBUG: [API badge-reader] Database created
NOTICE: [API badge-reader] [cn56-ex-sas] Connection ...
NOTICE: [API badge-reader] [cn56-revision] Connection ...
NOTICE: [API badge-reader] [cn56-revision] Connected
NOTICE: [API badge-reader] [cn56-ex-sas] Connected
NOTICE: [API badge-reader] [cn56-new] Connection ...
DEBUG: Init config done
NOTICE: API badge-reader starting...
WARNING: [API badge-reader] OnloadConfig Cannot Exec Non Existing Onload Action [/usr/src/packages/BUILD/ctl-lib/ctl-onload.c:38,OnloadConfig]
WARNING: [API badge-reader] Warning 1 raised during call to 'onload' section callback [/usr/src/packages/BUILD/ctl-lib/ctl-config.c:171,CtlConfigExec]
DEBUG: [API badge-reader] Database already open
DEBUG: [API badge-reader] Database created
INFO: API badge-reader started
NOTICE: API monitor starting...
INFO: API monitor started
INFO: HTTP server starting
NOTICE: Listening interface *:1234
NOTICE: [API badge-reader] [cn56-new] Connected

export PORT=1234
afb-binder --name=afb-badge-reader --port=${PORT} --workdir=./package --ldpaths=. -vvv
```

Now your application is reachable by HTTP or websocket through the port `${PORT}`.  

## IV - API usage

Badge Reader binding creates :

* general verb
* one api/verb by reader. By default each reader api/verb is prefixed by the Family-Reader uid
* Database api following CRUD (Create Read Update Delete) template

![devtools](./images/devtools.png)

### a. General API

#### 1. ping

```bash
$ afb-client --human 'localhost:1234/api' badge-reader ping
ON-REPLY 1:badge-reader/ping: OK
{
  "jtype":"afb-reply",
  "request":{
    "status":"success"
  },
  "response":"Pong=0"
}
```

#### 2. Info

```bash
$ afb-client --human 'localhost:1234/api' badge-reader info
ON-REPLY 1:badge-reader/info: OK
{
  "jtype":"afb-reply",
  "request":{
    "status":"success"
  },
  "response":{
    "metadata":{
      "uid":"badge-reader",
      "info":"Default config to use badge-reader binding",
      "version":"1.0"
    },
    "groups":[
      {
        "uid":"general",
        "info":"General command to send to the badge-reader binding",
        "verbs":[
          {
            "uid":"ping",
            "info":"Ping the binding",
            "verb":"ping"
          },
          {
            "uid":"info",
            "info":"Info of the binding",
            "verb":"info"
          },
          {
            "uid":"list",
            "info":"List all readers",
            "verb":"list"
          },
          {
            "uid":"subscribe",
            "info":"Subscribe to reader event",
            "verb":"subscribe"
          },
          {
            "uid":"unsubscribe",
            "info":"Subscribe to reader event",
            "verb":"unsubscribe"
          }
        ]
      },
      {
        "uid":"database",
        "info":"Administrate sqlite database",
        "verbs":[
          {
            "uid":"add_badge",
            "info":"Adding badge into database",
            "verb":"add_badge",
            "sample":[
              {
                "key":"11223344",
                "name":"username"
              }
            ]
          },
          {
            "uid":"get_badge",
            "info":"Get the badge from database",
            "verb":"get_badge",
            "sample":[
              {
                "key":"11223344"
              }
            ]
          },
          {
            "uid":"list_badge",
            "info":"List badge from database",
            "verb":"list_badge"
          },
          {
            "uid":"update_badge",
            "info":"update badge into database",
            "verb":"update_badge",
            "sample":[
              {
                "key":"11223344",
                "control":"000000"
              }
            ]
          },
          {
            "uid":"delete_badge",
            "info":"delete badge into database",
            "verb":"delete_badge",
            "sample":[
              {
                "key":"11223344"
              }
            ]
          }
        ]
      },
      {
        "uid":"cn56",
        "info":"CN56 Badge Reader TCP/UDP",
        "verbs":[
          {
            "uid":"cn56-ex-sas",
            "family":"cn56",
            "info":"Old badge reader principal entry",
            "verb":"cn56/cn56-ex-sas",
            "usage":{
              "action":[
                "write",
                "read",
                "activate",
                "error",
                "subscribe",
                "unsubscribe",
                "state",
                "reset"
              ],
              "data":{
                "sec":11,
                "blk":0,
                "len":4,
                "key":"key-a",
                "data":"test"
              }
            },
            "sample":[
              {
                "action":"write",
                "data":{
                  "sec":11,
                  "blk":0,
                  "len":4,
                  "key":"key-a",
                  "data":"test"
                }
              }
            ]
          },
          {
            "uid":"cn56-revision",
            "family":"cn56",
            "info":"Badge reader going to revision",
            "verb":"cn56/cn56-revision",
            "usage":{
              "action":[
                "write",
                "read",
                "activate",
                "error",
                "subscribe",
                "unsubscribe",
                "state",
                "reset"
              ],
              "data":{
                "sec":11,
                "blk":0,
                "len":4,
                "key":"key-a",
                "data":"test"
              }
            },
            "sample":[
              {
                "action":"write",
                "data":{
                  "sec":11,
                  "blk":0,
                  "len":4,
                  "key":"key-a",
                  "data":"test"
                }
              }
            ]
          },
          {
            "uid":"cn56-new",
            "family":"cn56",
            "info":"New badge reader",
            "verb":"cn56/cn56-new",
            "usage":{
              "action":[
                "write",
                "read",
                "activate",
                "error",
                "subscribe",
                "unsubscribe",
                "state",
                "reset"
              ],
              "data":{
                "sec":11,
                "blk":0,
                "len":4,
                "key":"key-a",
                "data":"test"
              }
            },
            "sample":[
              {
                "action":"write",
                "data":{
                  "sec":11,
                  "blk":0,
                  "len":4,
                  "key":"key-a",
                  "data":"test"
                }
              }
            ]
          }
        ]
      }
    ]
  }
}
```

#### 3. list

This verb list all readers configured and gives state information.

```bash
$ afb-client --human 'localhost:1234/api' badge-reader list
ON-REPLY 1:badge-reader/list: OK
{
  "jtype":"afb-reply",
  "request":{
    "status":"success"
  },
  "response":[
    {
      "uid":"cn56",
      "info":"CN56 Badge Reader TCP/UDP",
      "countReaders":3,
      "readers":[
        {
          "uid":"cn56-ex-sas",
          "info":"Old badge reader principal entry",
          "state":"connected"
        },
        {
          "uid":"cn56-revision",
          "info":"Badge reader going to revision",
          "state":"connected"
        },
        {
          "uid":"cn56-new",
          "info":"New badge reader",
          "state":"connected"
        }
      ]
    }
  ]
}

```

#### 4. Subscribe

The subscription will subscribe to all event comes from each reader configured! Each reader will possibly send 2 kinds of event:

* badge-reader/**good-badges** -> push when a badges is detected and verify all security configured. GIves a JSON in response:
  * **"reader"**: UID of the reader where comes the event.
  * **"badge"**: UUID of the badge detected.
  * **"name"**: Name of the badge owner stored into the database.
* badge-reader/**bad-badges**-> push when a badge is detected but doesn't match the security
  * **"reader"**: UID of the reader where comes the event.
  * **"badge"**: UUID of the badge detected.
  * **"error"**: Kind of the error, could be:
    * *"Bad Magic Key"* -> Means that the badge have proper keyA and KeyB but missing the secret data store into the specific block.
    * *"Bad key"* -> Means that the badges is not encrypted.
    * *"cloned"* -> Means the badges has been cloned. Tha data control is wrong.
    * *"Not found"* -> Means that the badge's UUID has not been found into the binding database.

```bash
$ afb-client --human 'ws://localhost:1234/api'
badge-reader subscribe
ON-REPLY 1:badge-reader/subscribe: OK
{
  "jtype":"afb-reply",
  "request":{
    "status":"success"
  },
  "response":"Subscribed to all event"
}
```

Events caught:

```bash
ON-EVENT badge-reader/good-badges:
{
  "jtype":"afb-event",
  "event":"badge-reader/good-badges",
  "data":{
    "reader":"cn56-new",
    "badge":"16b15a1f",
    "name":"user"
  }
}
ON-EVENT badge-reader/bad-badges:
{
  "jtype":"afb-event",
  "event":"badge-reader/bad-badges",
  "data":{
    "reader":"cn56-ex-sas",
    "badge":"00000000",
    "error":"Bad magic key"
  }
}
ON-EVENT badge-reader/bad-badges:
{
  "jtype":"afb-event",
  "event":"badge-reader/bad-badges",
  "data":{
    "reader":"cn56-ex-sas",
    "badge":"16b15a1f",
    "error":"Bad magic key"
  }
}
```

#### 5. Unsubscribe

```bash
badge-reader unsubscribe
ON-REPLY 2:badge-reader/unsubscribe: OK
{
  "jtype":"afb-reply",
  "request":{
    "status":"success"
  },
  "response":"Unsubscribed to all event"
}
```

### b. Reader Action

For each reader, we have a verb. For example for the reader `cn56-ex-sas` which is in the `cn56` family, it will get the following verb: `cn56/cn56-ex-sas`. For each reader, yuo will have this list of possible action:

#### 1. write

/!\ Need to be update with new auto read mode

```bash
$ afb-client --human 'ws://localhost:1234/api' badge-reader cn56/cn56-ex-sas '{"action":"write","data":{"sec":9,"blk":1,"len":4,"key":"default-A","data":"test"}}'
ON-REPLY 1:badge-reader/cn56/cn56-ex-sas: ERROR
{
  "jtype":"afb-reply",
  "request":{
    "status":"action error",
    "info":"Failed to apply action write for reader cn56/cn56-ex-sas"
  }
}
```

#### 2. read

/!\ Need to be update with new auto read mode

```bash
$ afb-client --human 'ws://localhost:1234/api' badge-reader cn56/cn56-ex-sas '{"action":"read","data":{"sec":9,"blk":1,"len":4,"key":"default-A"}}'
ON-REPLY 1:badge-reader/cn56/cn56-ex-sas: ERROR
{
  "jtype":"afb-reply",
  "request":{
    "status":"action error",
    "info":"Failed to apply action write for reader cn56/cn56-ex-sas"
  }
}
```

#### 3. activate

Send the activate reader command. In CN56 example, will set LED to green for 5 seconds, and open de door for 4 seconds.

```bash
$ afb-client --human 'ws://localhost:1234/api' badge-reader cn56/cn56-ex-sas '{"action":"activate"}'
ON-REPLY 1:badge-reader/cn56/cn56-ex-sas: OK
{
  "jtype":"afb-reply",
  "request":{
    "status":"success"
  }
}
```

#### 4. error

Send the error reader command. In CN56 example blink the red LED for 1 second

```bash
$ afb-client --human 'ws://localhost:1234/api' badge-reader cn56/cn56-ex-sas '{"action":"error"}'
ON-REPLY 1:badge-reader/cn56/cn56-ex-sas: OK
{
  "jtype":"afb-reply",
  "request":{
    "status":"success"
  }
}
```

### c. Databases

#### 1. add_badge

Add a UUid badge on a database

```bash
$ afb-client --human 'ws://localhost:1234/api' badge-reader add_badge '{"key":"tata","name":"username"}'
ON-REPLY 1:badge-reader/add_badge: OK
{
  "jtype":"afb-reply",
  "request":{
    "status":"success"
  },
  "response":"Badge=tata has been added to database"
}
```

#### 2. get_badge

Get badge information from database

```bash
$ afb-client --human 'ws://localhost:1234/api' badge-reader get_badge '{"key":"tata"}'
ON-REPLY 1:badge-reader/get_badge: OK
{
  "jtype":"afb-reply",
  "request":{
    "status":"success"
  },
  "response":"Keys [tata] : name = username, control =test!"
}
```

#### 3. list_badge

List all UUID badges from database

```bash
$ afb-client --human 'ws://localhost:1234/api' badge-reader list_badge
ON-REPLY 1:badge-reader/list_badge: OK
{
  "jtype":"afb-reply",
  "request":{
    "status":"success"
  },
  "response":{
    "list":[
      "16b15a1f",
      "26d0701f",
      "toto",
      "tata"
    ]
  }
}
```

#### 4. update_badge

Update the control column of a badge in database

```bash
$ afb-client --human 'ws://localhost:1234/api' badge-reader update_badge '{"key":"tata","control":"20210817"}'
ON-REPLY 1:badge-reader/update_badge: OK
{
  "jtype":"afb-reply",
  "request":{
    "status":"success"
  },
  "response":"Keys [tata] :has been update!"
}
```

#### 5. delete_badge

Delete a badge from the database

```bash
$ afb-client --human 'ws://localhost:1234/api' badge-reader delete_badge '{"key":"tata"}'
ON-REPLY 1:badge-reader/delete_badge: OK
{
  "jtype":"afb-reply",
  "request":{
    "status":"success"
  },
  "response":"Badge=tata has been deleted from database"
}
```

## Link

* [ReadMe](../README.md)
* [Architecture](1-Architecture.md)
* [Installation](2-Installation.md)
* [Security](4-security.md)
