# CN56

This document relate the study of the CN56 badge reader. :smile:

## Table of Contents

* [Cmd / response frame](#i---cmd--response-frame)
* [Mifare Encryption](#ii---mifare-encryption)
* [Mifare Read](#iii---mifare-read)
* [Mifare Write](#iv---mifare-write)
* [Security](#v---security)

---

## I - Cmd / response frame

### a) Frame to be send

| STX | SEQ | DADD | CMD | DATA_LENGTH | TIME | DATA[0..n] | BCC | ETX |
|:---:|:---:|:----:|:---:|:-----------:|:----:|:----------:|:---:|:---:|

### b) Frame in response

| STX | SEQ | DADD | DATA_LENGTH | STATUS | DATA[0..n] | BCC | ETX |
|:---:|:---:|:----:|:-----------:|:------:|:----------:|:---:|:---:|

---

## II - Mifare encryption

To be able to encrypt a sector on a Mifare badge, we need to send some command in first step, like authentication, and then send the command to override all keys

### a) Reverse engineering

Thanks to Wireshark we can study all frame sending by the CN56tools to the reader:

#### For sector 8

| Command Name | Cmd | Data | Response |
|:------------:|:----|:-----|:--------:|
| PN_InListPassive | **0x3d** | 0x00 0x02 0x00 | 0x01 0x01 0x00 0x04 0x08 0x04 **0x16 0xb1 0x5a 0x1f** |
| PN_SelectTarget | **0x3e** | 0x01 | none |
| LoadKey | **0x45** | 0xff 0xff 0xff 0xff 0xff 0xff | none |
| Authentication | **0x40** | 0x60 **0x16 0xb1 0x5a 0x1f** 0x20 | none |
| Read | **0x41** | 0x21 0x01 | 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 |
| Modify Key | **0x42** | 0x23 0x01 0xaa 0xbb 0xcc 0xdd 0xee 0xff 0xff 0x07 0x80 0x00 0xff 0xff 0xff 0xff 0xff 0xff | none |

#### For sector 9

| Command Name | Cmd | Data | Response |
|:------------:|:----|:-----|:--------:|
| PN_InListPassive | **0x3d** | 0x00 0x02 0x00 | 0x01 0x01 0x00 0x04 0x08 0x04 **0x16 0xb1 0x5a 0x1f** |
| PN_SelectTarget | **0x3e** | 0x01 | none |
| LoadKey | **0x45** | 0xff 0xff 0xff 0xff 0xff 0xff | none |
| Authentication | **0x40** | 0x60 **0x16 0xb1 0x5a 0x1f** 0x24 | none |
| Read | **0x41** | **0x24** 0x01 | 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 0x00 |
| Modify Key | **0x42** | 0x27 0x01 0xff 0xff 0xff 0xff 0xff 0xff 0xff 0x07 0x80 0x00 0x01 0x02 0x03 0x04 0x05 0x06 | none |

#### PN_InListPassive

In response of this command we get the id of the Badge

#### Authentication

* We need to provides the ID of the badge to authenticate
* 0x60 can be for using KeyA to read the value
* 0x21=32 block 0 of sector 8

#### Modify Key

* byte 1 = Block (0x23 = 35 = 8*4 + 3 = block 3 of sector 8)
* byte 3 to 8 = KeyA
* byte 9 to 12 = Acl
* byte 13 to 18 = KeyB

### b) Authentication

TO be able to Encrypt or Read a sector after an encryption, we need to authenticate a badge following this steps:

1. **pn_InListPassive**: command = **0x3d** -> It will gives in response the Id of the badge, useful for the authentication step
2. **pn_selectTarget**: command = **0x3e**
3. **loadKey**: command = **0x45** -> give 6 bits for the key
4. **Authenticate**: command = **0x40** -> give the badge ID

### c) Modify Key

To modify a sector's key:

| Command Name | Cmd | Data[18] | Response |
|:------------:|:----|:-----|:--------:|
| Modify Key | **0x42** | byte 1 n° block<br />byte 2 0x01<br /> bytes 3->9 Key A<br />bytes 10->13 acl<br />bytes 14->18 KeyB | none |

### d) Acls keys managment

See this [file](mifare_1.md#Access-bits) eplains how to construct the Acl token. For our case we will need to encrypt the badge as follow:

* Key B will be used to
  * write the **block 3** (read KeyA  KeyB and acl values)
  * Read / Write on data section **block 0**, **block 1** and **blok 2**
* Key A will be used to
  * Read acl on **block 3**
  * Read data section on **block 1** and **block 2**
  * Read / Write data section on **block 0**

According this information, here is the tab to create the acl:

|   | C1 | C2 | C3 |
|:--|:--:|:--:|:--:|
|blk 0 | 0 | 0 | 0 |
|blk 1 | 1 | 0 | 0 |
|blk 2 | 1 | 0 | 0 |
|blk 3 | 0 | 1 | 1 |

Acl -> **0x79 0x67 0x88**

---

## III - Mifare read

### a) one-shot command

For read a block into a sector we will need to load a key (according to the acl) and authenticate:

| Command Name | Cmd | Data | Response |
|:------------:|:----|:-----|:--------:|
| PN_InListPassive | **0x3d** | 0x00 0x02 0x00 | 0x01 0x01 0x00 0x04 0x08 0x04 [**UUID badge on 4 bytes**] |
| PN_SelectTarget | **0x3e** | **N° blk** | none |
| LoadKey | **0x45** | [**Key on 6 bytes**] | none |
| Authentication | **0x40** | [**0x60 for keyA / 0x61 for key b**] [**UUID badge on 4 bytes**] [**block0 of the sector**] | none |
| Read | **0x41** | **0x24** | **N° blk** | **16 bits of the sector** |

### b) multi command

If we have to send a succession of command which use the same key into the same sector: The authentication part has to be done just ine time at the beginning. For example if you want the read + write + read to verify, we can do:

1. **pn_InListPassive**: command = **0x3d** -> It will gives in response the Id of the badge, useful for the authentication step
2. **pn_selectTarget**: command = **0x3e**
3. **loadKey**: command = **0x45** -> give 6 bits for the key
4. **Authenticate**: command = **0x40** -> give the badge ID
5. **Read**: command = **0x41** -> Read the block
6. **Write**: command = **0x42** -> Write the data to the block
7. **Read**: command = **0x41** -> Verify the data wrote

---

## IV - Mifare write

For write a block into a sector we will need to load a key (according to the acl) and authenticate:

| Command Name | Cmd | Data | Response |
|:------------:|:----|:-----|:--------:|
| PN_InListPassive | **0x3d** | 0x00 0x02 0x00 | 0x01 0x01 0x00 0x04 0x08 0x04 [**UUID badge on 4 bytes**] |
| PN_SelectTarget | **0x3e** | **N° blk** | none |
| LoadKey | **0x45** | [**Key on 6 bytes**] | none |
| Authentication | **0x40** | [**0x60 for keyA / 0x61 for key b**] [**UUID badge on 4 bytes**] [**block0 of the sector**] | none |
| Write | **0x42** | **0x24** | **N° blk** | **16 bits of the sector** |

---

## V - Security

To increase the security of the CN56 badge reader, it is better to apply theses 3 next points

### a) Keys & acls

* Aplly different keys by sectors
* Use Key B for write data (=private key)
* Use Key A for only Read data (=public key)

### b) encrypted data

Data wrote on the badge has to be encrypted with a key **construct by the uuid of the badge**. In this case, we can ensure to not read a cloned badged (If the uuid has not been overwritten). Each badges will have its own encrypted data.

### c) blocks begins by 0

Some brut-force hacking software test all keys and check if a block returns values different by 0. SO when we will write data on a sector's block, it will be better write 0x00 for the sixth first bytes.

### d) Updated data

To ensure to not have cloned data, it will be useful to update a data on a specific block each day.

Workflow:

* Worker comes at the morning and badges
* Badge-reader catch the code of the last day, overwrite it with the nex code of the day
* Badge-reader update its own database and put the uuid of the badge with the new code
* If a cloned badge comes, the badge-reader will see for the second time the code of the last day -> means that we have a cloned badges. We will be able to notify it !
